@@ -532,12 +532,16 @@ testScans :: Test
 testScans = testCase "scans" $ do
     set "key" "value"       >>=? Ok
     scan cursor0            >>=? (cursor0, ["key"])
+    scanOpts cursor0 sOpts1 >>=? (cursor0, ["key"])
+    scanOpts cursor0 sOpts2 >>=? (cursor0, [])
     sadd "set" ["1"]        >>=? 1
     sscan "set" cursor0     >>=? (cursor0, ["1"])
     hset "hash" "k" "v"     >>=? True
     hscan "hash" cursor0    >>=? (cursor0, [("k", "v")])
     zadd "zset" [(42, "2")] >>=? 1
     zscan "zset" cursor0    >>=? (cursor0, [("2", 42)])
+    where sOpts1 = defaultScanOpts { scanMatch = Just "k*" }
+          sOpts2 = defaultScanOpts { scanMatch = Just "not*"}
 
 testZrangelex ::Test
 testZrangelex = testCase "zrangebylex" $ do
